/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

using System;
using System.Collections.Generic;
using Tizen.NUI.BaseComponents;
using Tizen.NUI;
using Tizen.NUI.UIComponents;

namespace NUISample.FlexContainerSample
{

    // Class for Style variables
    class StyleCollection
    {
        // TextVisual for text style
        public static TextVisual TitleText;
        public static TextVisual ButtonText;
        
        // Void Property
        public static PropertyMap Void;

        // BorderVisual for button's border
        public static BorderVisual ButtonBoder;

        // Color variables for Theme of this sample application
        public static Color LightColor;
        public static Color HighlightColor;
        public static Color LightGray;
        public static Color Gray;
        public static void Init()
        {
            // Initialize Color variables
            HighlightColor = new Color(130.0f / 255.0f, 207.0f / 255.0f, 221.0f / 255.0f, 1);
            LightColor = new Color(61.0f / 255.0f, 183.0f / 255.0f, 203.0f / 255.0f, 1);
            LightGray = new Color(0.8f, 0.8f, 0.8f, 1);
            Gray = new Color(0.5f, 0.5f, 0.5f, 1);

            // Initialize Title Text property.
            Void = new PropertyMap();
            TitleText = new TextVisual();
            TitleText.FontFamily = "Breeze Sans Medium";
            TitleText.TextColor = Color.White;
            TitleText.Text = " ";
            TitleText.PointSize = 15;

            // Initialize RadioButton Text property.
            ButtonText = new TextVisual();
            ButtonText.FontFamily = "Breeze Sans Light";
            ButtonText.TextColor = Color.White;
            ButtonText.Text = " ";
            ButtonText.HorizontalAlignment = HorizontalAlignment.Begin;
            ButtonText.VerticalAlignment   = VerticalAlignment.Center;
            ButtonText.PointSize = 10;

            // Initialize RadioButton Border property.
            ButtonBoder = new BorderVisual();
            ButtonBoder.Color = Color.White;
            ButtonBoder.BorderSize = 3f;
        }
    }

    internal class Example : NUIApplication
    {
        Window window;
        TextLabel titleBar;
        FlexContainer bodyLayout;
        TableView menuLayout;
        FlexContainer contentLayout;

        readonly static uint COUNT_MENU_COLUMNS = 5;

        // String Array for Property Types
        readonly static String[] FLEX_PROPERTY_TYEP =
        {
            "FlexDirectionType",
            "WrapType",
            "ContentDirection",
            "JustifyContent",
            "AlignContent"
        };

        // String Array for Property Names
        readonly static String[][] FLEX_PROPERTY_NAME =
        {
            new String[] {"Column", "ColumnReverse", "Row", "RowReverse" },
            new String[] {"NoWrap", "Wrap" },
            new String[] {"Inherit", "LTR", "RTL" },
            new String[] {"JustifyFlexStart", "JustifyCenter", "JustifyFlexEnd", "JustifySpaceAround", "JustifySpaceBetween"},
            new String[] {"AlignAuto", "AlignFlexStart", "AlignCenter", "AlignFlexEnd", "AlignStretch"}
        };

        // Dictionary for matching Property and String
        readonly static Dictionary<String, Enum> PROPERTY_DIC = new Dictionary<String, Enum>()
        {
            {"Column",              FlexContainer.FlexDirectionType.Column },
            {"ColumnReverse",       FlexContainer.FlexDirectionType.ColumnReverse },
            {"Row",                 FlexContainer.FlexDirectionType.Row },
            {"RowReverse",          FlexContainer.FlexDirectionType.RowReverse },

            {"Wrap",                FlexContainer.WrapType.Wrap },
            {"NoWrap",              FlexContainer.WrapType.NoWrap },

            {"Inherit",             FlexContainer.ContentDirectionType.Inherit },
            {"LTR",                 FlexContainer.ContentDirectionType.LTR },
            {"RTL",                 FlexContainer.ContentDirectionType.RTL },

            {"JustifyFlexStart",    FlexContainer.Justification.JustifyFlexStart },
            {"JustifyCenter",       FlexContainer.Justification.JustifyCenter },
            {"JustifyFlexEnd",      FlexContainer.Justification.JustifyFlexEnd },
            {"JustifySpaceAround",  FlexContainer.Justification.JustifySpaceAround },
            {"JustifySpaceBetween", FlexContainer.Justification.JustifySpaceBetween },

            {"AlignAuto",           FlexContainer.Alignment.AlignAuto },
            {"AlignFlexStart",      FlexContainer.Alignment.AlignFlexStart },
            {"AlignCenter",         FlexContainer.Alignment.AlignCenter },
            {"AlignFlexEnd",        FlexContainer.Alignment.AlignFlexEnd },
            {"AlignStretch",        FlexContainer.Alignment.AlignStretch }
        };



        private void Initialize()
        {
            //Initialize Style variable (VisualProperty and Colors) for Application
            StyleCollection.Init();

            // Get Stage instance.
            // Stage is only one for each application.
            window = Window.Instance;
            window.KeyEvent += OnKeyEvent;

            // Set Stage's background color using Color class constant.
            // Color class has constants for basic colors.
            window.BackgroundColor = Color.White;

            // Create Title Bar, BodyLayout and set Position and attributes.
            CreateLayout();
            titleBar.Text = " Flex Container Test";

            // Add Title Bar and Body Layout to Default Layer.
            // Stage has Default Layer to contain ordinary contents.
            window.GetDefaultLayer().Add(titleBar);
            window.GetDefaultLayer().Add(bodyLayout);

            // Create Menu Layout
            // it based on TableView, each column has a RadioGroup
            menuLayout = CreateMenu(0.33f);
            bodyLayout.Add(menuLayout);
            FocusManager.Instance.SetCurrentFocusView(View.DownCast(menuLayout.GetChildAt(new TableView.CellPosition(1, 0)).GetChildAt(0)));

            // Create ContentsContainer
            // It will show children 
            contentLayout = new FlexContainer();
            contentLayout.BackgroundColor = StyleCollection.LightGray;
            contentLayout.Flex = 0.66f;
            contentLayout.ClippingMode = ClippingModeType.ClipChildren;
            bodyLayout.Add(contentLayout);

            Random rd = new Random();
            for (int i = 0; i < 40 ; i++)
            {
                // Create FlexContainer Items
                TextLabel flexitem = new TextLabel("" + i);
                flexitem.PointSize = 10;
                flexitem.HorizontalAlignment = HorizontalAlignment.Center;
                flexitem.VerticalAlignment   = VerticalAlignment.Center;
                flexitem.FlexMargin = new Vector4(5f, 5f, 5f, 5f);

                // Set Item Background Color
                flexitem.BackgroundColor = new Color((rd.Next() % 55 + 200) / 255f,
                                                     (rd.Next() % 55 + 200) / 255f,
                                                     (rd.Next() % 55 + 200) / 255f,
                                                     1.0f);

                // Set Item Size and ResizePolicy
                flexitem.HeightResizePolicy = ResizePolicyType.Fixed;
                flexitem.WidthResizePolicy  = ResizePolicyType.Fixed;
                flexitem.Size2D = window.Size / 7;  
                contentLayout.Add(flexitem);

            }
        }
        
        private void CreateLayout()
        {
            // Create Title Bar
            titleBar = new TextLabel();

            // View can inherit parent's size via setting SizeModeFactor.
            // First value is factor for width.
            // Second value is factor for height.
            titleBar.WidthResizePolicy = ResizePolicyType.FillToParent;
            titleBar.HeightResizePolicy = ResizePolicyType.SizeRelativeToParent;
            titleBar.SizeModeFactor = new Vector3(0.0f, 0.1f, 0.0f);
            
            //Set Title Text attribute (Alignment, Color, TextSize)
            titleBar.HorizontalAlignment = HorizontalAlignment.Begin;
            titleBar.VerticalAlignment = VerticalAlignment.Center;
            titleBar.FontFamily = StyleCollection.TitleText.FontFamily;
            titleBar.TextColor = StyleCollection.TitleText.TextColor;
            titleBar.BackgroundColor = StyleCollection.LightColor;
            titleBar.PointSize = StyleCollection.TitleText.PointSize;

            // Create Body Layout
            bodyLayout = new FlexContainer();
            bodyLayout.WidthResizePolicy = ResizePolicyType.FillToParent;
            bodyLayout.HeightResizePolicy = ResizePolicyType.SizeRelativeToParent;
            bodyLayout.SizeModeFactor = new Vector3(0.0f, 0.9f, 0.0f);
            bodyLayout.PositionUsesAnchorPoint = true;

            // BodyLayout is bottom of window
            bodyLayout.AnchorPoint = AnchorPoint.BottomCenter;
            bodyLayout.ParentOrigin = ParentOrigin.BottomCenter;

            // Vertical Align the contents in BodyLayout.
            bodyLayout.FlexDirection = FlexContainer.FlexDirectionType.Column;
        }

        private TableView CreateMenu(float RelativeSize)
        {
            TableView menu = new TableView(2, COUNT_MENU_COLUMNS);
            menu.Size = new Size(window.Size.Width, window.Size.Height * RelativeSize, 0.0f);
            
            // Set Rows Height
            // Row 0 : TextLabel for descriptor FlexContainer attribute
            // Row 1 : RadioButton group
            menu.SetRelativeHeight(0, 0.1f);
            menu.SetRelativeHeight(1, 0.9f);

            // Set Column Width
            // each property names has different length, 
            // So it need to modify column width for show full text 
            // sum of relative width must be 1.0f.
            menu.SetRelativeWidth(0, 0.22f);
            menu.SetRelativeWidth(1, 0.15f);
            menu.SetRelativeWidth(2, 0.15f);
            menu.SetRelativeWidth(3, 0.28f);
            menu.SetRelativeWidth(4, 0.2f);
            
            // Create Controls in each column
            for (uint i = 0;i < COUNT_MENU_COLUMNS;i++)
            {
                TextLabel typename = new TextLabel(FLEX_PROPERTY_TYEP[i]);
                typename.PointSize = 8f;
                typename.TextColor = StyleCollection.Gray;
                typename.HorizontalAlignment = HorizontalAlignment.Center;
                menu.AddChild(typename, new TableView.CellPosition(0, i));
                menu.SetCellAlignment(new TableView.CellPosition(0, i), HorizontalAlignmentType.Left, VerticalAlignmentType.Top);

                
                FlexContainer radioGroup = new FlexContainer();
                radioGroup.BackgroundColor = Color.Red;
                radioGroup.FlexDirection = FlexContainer.FlexDirectionType.Column;

                radioGroup.Size = new Size(menu.GetRelativeWidth(i) * menu.SizeWidth, menu.SizeHeight * 0.88f, menu.Size.Depth);

                menu.AddChild(radioGroup, new TableView.CellPosition(1, i));
                menu.SetCellAlignment(new TableView.CellPosition(1, i), HorizontalAlignmentType.Left, VerticalAlignmentType.Bottom);
                
                for (int j = 0; j < FLEX_PROPERTY_NAME[i].Length; j++)
                {
                    // Create new RadioButton
                    RadioButton rb = new RadioButton();
                    if (j == 0)
                    {
                        rb.Selected = true;
                    }

                    // Set Radiobutton size property
                    rb.Flex = 1.0f / FLEX_PROPERTY_NAME[i].Length;
                    rb.WidthResizePolicy = ResizePolicyType.FillToParent;

                    // Set Text on RadioButton
                    rb.Label = StyleCollection.ButtonText.OutputVisualMap;
                    rb.LabelText = FLEX_PROPERTY_NAME[i][j];
                    rb.Name = FLEX_PROPERTY_NAME[i][j];
                    
                    // Set Color of RadioButton
                    rb.UnselectedColor  = StyleCollection.LightColor;
                    rb.SelectedColor    = StyleCollection.HighlightColor;

                    // Set VisualProperty
                    rb.Background = StyleCollection.ButtonBoder.OutputVisualMap;
                    rb.UnselectedVisual = StyleCollection.Void;
                    rb.SelectedVisual = StyleCollection.Void;

                    // Add EventHandler RadioButton
                    rb.StateChanged += SelectProperty;

                    // Add to radio group
                    radioGroup.Add(rb);
                    
                }

            }

            return menu;
        }

        private bool SelectProperty(object sender, EventArgs e)
        {
            RadioButton bt = sender as RadioButton;

            switch (bt.Name)
            {
                /*
                 * 1.FlexDirection
                 * : Enumeration for the direction of the main axis in the flex container {Column, Column Reverse, Row, Row Reverse}
                 *   This determines the direction that flex items are laid out in the flex container.
                 *   
                 *   Column        - The flexible items are displayed vertically as a column
                 *   ColumnReverse - The flexible items are displayed vertically as a column, but in reverse order
                 *   Row           - The flexible items are displayed horizontally as a row
                 *   RowReverse    - The flexible items are displayed horizontally as a row, but in reverse order
                 */
                case "Column":
                case "ColumnReverse":
                case "Row":
                case "RowReverse":
                    contentLayout.FlexDirection = (FlexContainer.FlexDirectionType)PROPERTY_DIC[bt.Name];
                    break;
                /*
                 * 2. FlexWrap
                 * : Enumeration for the wrap type of the flex container when there is no enough room for
                 *   all the items on one flex line
                 *   
                 *   Wrap   - Flex items laid out in multiple lines if needed
                 *   NoWrap - Flex items laid out in single line (shrunk to fit the flex container along the main axis)
                 */
                case "NoWrap":
                case "Wrap":
                    contentLayout.FlexWrap = (FlexContainer.WrapType)PROPERTY_DIC[bt.Name];
                    break;

                /*
                 * 3. ContentDirection
                 * : Enumeration for the primary direction in which content is ordered in the flex container
                 * 
                 *   and on which sides the start and end are.
                 *   Inherit  - Inherits the same direction from the parent
                 *   LTR      - Contents is collocated from Left to right
                 *   RTL      - Contents is collocated from right to left
                 */
                case "Inherit":
                case "LTR":
                case "RTL":
                    contentLayout.ContentDirection = (FlexContainer.ContentDirectionType)PROPERTY_DIC[bt.Name];
                    break;

                /*
                 * 4. Justify Contents
                 * : Enumeration for the alignment of the flex items when the items do not use all available
                 * 
                 *   JustifyFlexStart    - Items are positioned at the end of the container
                 *   JutsifyCenter       - Items are positioned at the beginning of the container
                 *   JustifyFlexEnd      - Items are positioned at the center of the container
                 *   JustifySpaceAround  - Items are positioned with equal space between the lines
                 *   JustifySpaceBetween - Items are positioned with equal space before, between, and after the lines
                 */
                case "JustifyFlexStart":
                case "JutsifyCenter":
                case "JustifyFlexEnd":                
                case "JustifySpaceAround":
                case "JustifySpaceBetween":
                    contentLayout.JustifyContent = (FlexContainer.Justification)PROPERTY_DIC[bt.Name];
                    break;

                /*
                 * 5. Align Contents
                 * : Enumeration for the alignment of the flex items or lines when the items or lines do not
                 *   use all the available space on the cross-axis.
                 *   
                 *   AlignAuto      - Inherits the same alignment from the parent (only valid for "alignSelf" property)
                 *   AlignFlexStart - At the beginning of the container
                 *   AlignCenter    - At the center of the container
                 *   AlignFlexEnd   - At the end of the container
                 *   AlignStretch   - Stretch to fit the container
                 */
                case "AlignAuto":
                case "AlignFlexStart":
                case "AlignCenter":
                case "AlignFlexEnd":
                case "AlignStretch":
                    contentLayout.AlignContent = (FlexContainer.Alignment)PROPERTY_DIC[bt.Name];
                    break;

            }

            return true;

        }

        private PushButton CreateMenuButton(Size size)
        {
            // Create Push Button Instance           
            PushButton button = new PushButton();

            // Set Button Color
            button.UnselectedColor = new Color(43.0f / 255.0f, 145.0f / 255.0f, 175.0f / 255.0f, 1.0f);
            button.SelectedColor = new Color(43.0f / 255.0f, 195.0f / 255.0f, 175.0f / 255.0f, 1.0f);
            
            // Focusable option Enable
            // This Option make to move focusing by Keyboard/Remote Controller input
            button.Focusable = true;           

            // Set button size
            button.Size = size;

            // Set Text Style for visual
            // in this case, set Font Size and Alignment
            TextVisual textvisual = new TextVisual();
            textvisual.PointSize = 7;
            textvisual.Text = ".";
            textvisual.HorizontalAlignment = HorizontalAlignment.Center;
            textvisual.VerticalAlignment = VerticalAlignment.Center;

            //Apply Text style to TextLabel of PushButton.
            button.Label = textvisual.OutputVisualMap;

            return button;

        }

        // Callback for Window Key Event
        private void OnKeyEvent(object source, Window.KeyEventArgs e)
        {
            if (e.Key.State == Key.StateType.Up)
            {
                if (e.Key.KeyPressedName.CompareTo("Escape") == 0 || e.Key.KeyPressedName.CompareTo("XF86Back") == 0)
                {
                    Exit();
                }
            }
        }

        protected override void OnCreate()
        {
            base.OnCreate();
            Initialize();            
        }

        [STAThread]
        private static void Main(string[] args)
        {
            Example example = new Example();
            example.Run(args);
        }
    }

}
